#include <QDropEvent>
#include "mainwindow.h"
#include "ui_mainwindow.h"

/* ===================================================================================================
   ==                                       INITIAL SETUP                                           ==
   =================================================================================================== */

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    // Setup the help text stuff.
    ui->textHelp->setText("Befuddler is tool used to change the signature and extension of files \r\n"
                          "so that they can evade annoyances such as over-zealous email filters. \r\n"
                          "\r\n"
                          "It works by reversing the order of the first ten bytes of the file and \r\n"
                          "applying a ROT-13 scheme to the file extension, if present. If less than\r\n"
                          "ten bytes are in the file, their order won't be reversed.\r\n"
                          "\r\n"
                          "Drag and drop files in this box to befuddle/defuddle files.");

    // Resize the application with the help info in there.
    this->updateGeometry();
    this->setMaximumSize(this->sizeHint());
    this->setMinimumSize(this->sizeHint());

    // Hide the help info again.
    ui->textHelp->setVisible(false);
    ui->iconHelp->setVisible(false);
    ui->textDefault->setVisible(true);

    // Initialize the variables.
    this->helpActive = false;
    this->befuddledMessage = 0;

    // Make the drag 'n drop widget accept drops.
    ui->dropArea->setAcceptDrops(true);

}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

/* ===================================================================================================
   ==                                         HELP BUTTON                                           ==
   =================================================================================================== */

void MainWindow::on_buttonHelp_toggled(bool checked)
{
    // Show/hide the help information
    if (helpActive == false) {
        ui->textDefault->setVisible(false);
        ui->textHelp->setVisible(true);
        ui->iconHelp->setVisible(true);
        helpActive = true;
    } else {
        ui->textDefault->setVisible(true);
        ui->textHelp->setVisible(false);
        ui->iconHelp->setVisible(false);
        helpActive = false;
    }
}

/* ===================================================================================================
   ==                                   DRAG 'N DROP HANDLING                                       ==
   =================================================================================================== */

void MainWindow::dragEnterEvent(QDragEnterEvent *event)
{
    ui->textDefault->setText("Drop File to Befuddle");
    ui->dropArea->setBackgroundRole(QPalette::Highlight);

    event->acceptProposedAction();
    emit changed(event->mimeData());
}

void MainWindow::dragMoveEvent(QDragMoveEvent *event)
{
    ui->textDefault->setText("Drop File to Befuddle");
    ui->dropArea->setBackgroundRole(QPalette::Highlight);
}

void MainWindow::dragLeaveEvent(QDragLeaveEvent *event)
{
    ui->textDefault->setText("Drag 'n Drop Files Here");
    ui->dropArea->setBackgroundRole(QPalette::Window);
}

void MainWindow::dropEvent(QDropEvent *event)
{
    ui->textDefault->setText("Drag 'n Drop Files Here");
    ui->dropArea->setBackgroundRole(QPalette::Window);

    const QMimeData *mimeData = event->mimeData();

    if (mimeData->hasUrls()) {

        // Extract the location of the file(s)
        QList<QUrl> urlList = mimeData->urls();

        // Process each file.
        int filesWithErrors = 0;
        for (int i = 0; i < urlList.size(); i++) {
            QString url = urlList.at(i).path().mid(1);
            filesWithErrors = beffudleFile(url, filesWithErrors);
        }

        // Report error/success to user
        if (filesWithErrors > 0) {
            QString fileCount;
            ui->textDefault->setText("Could Not Befuddle " + fileCount.setNum(filesWithErrors, 10) + " File(s).");
            befuddledMessage = 0;
        } else if (befuddledMessage == 0) {
            ui->textDefault->setText("Befuddling Succesful!");
            befuddledMessage++;
        } else if (befuddledMessage == 1) {
            ui->textDefault->setText("More Befuddling Success!");
            befuddledMessage = 0;
        }

    }
}

/* ===================================================================================================
   ==                                   FILE BEFUDDLING LOGIC                                       ==
   =================================================================================================== */

int MainWindow::beffudleFile(QString filePath, int errorCount) {

    // Set target file.
    QFile file(filePath);

    // If file is greater than 10 bytes long, reverse the order of the
    // first ten bytes to thwart file signature sniffing.
    if (file.size() > 10) {

        // Set memory aside to load in bytes.
        QByteArray data;
        QByteArray reversedData;

        // Open the file. Abort if it fails.
        if (!file.open(QIODevice::ReadWrite))
            return errorCount+1;

        // Load in the first ten bytes.
        data = file.read(10);

        // If the data can not be fully read, abort.
        if (data.size() < 10)
            return errorCount+1;

        // Reverse the first ten bytes.
        for (int i = 9; i > -1; i--) {
            reversedData.append(data.at(i));
        }

        // Write the reversed bytes.
        file.seek(0);
        file.write(reversedData);

        // Close file.
        file.close();
    }

    // Obtain the file extension.
    QString fileExtension = filePath.split("/", QString::KeepEmptyParts).last().
                            split(".", QString::KeepEmptyParts).last();

    // If the file has a file extension, apply ROT-13
    // to it.
    if (fileExtension.size() > 0) {

        // ROT-13 the extension
        fileExtension = this->rot13(fileExtension);

        // Change the file name to use the new extension
        QString newFilePath(filePath);
        newFilePath.chop(fileExtension.size());
        newFilePath.append(fileExtension);
        file.rename(filePath, newFilePath);
    }

    return errorCount;
}

/* ===================================================================================================
   ==                                      ROT-13 FUNCTION                                          ==
   =================================================================================================== */

QString MainWindow::rot13(QString input)
{
    QString output;
    for (int i = 0; i < input.size(); i++) {
        char letter = input.at(i).toAscii();
        if ('A' <= letter && letter <= 'M') letter += 13;
        else if ('N' <= letter && letter <= 'Z') letter -= 13;
        else if ('a' <= letter && letter <= 'm') letter += 13;
        else if ('n' <= letter && letter <= 'z') letter -= 13;
        output.append(letter);
    }
    return output;
}
